/* automatically generated from xfce4-session-settings.ui */
#ifdef __SUNPRO_C
#pragma align 4 (xfce4_session_settings_ui)
#endif
#ifdef __GNUC__
static const char xfce4_session_settings_ui[] __attribute__ ((__aligned__ (4))) =
#else
static const char xfce4_session_settings_ui[] =
#endif
{
  "<?xml version=\"1.0\" encoding=\"UTF-8\"?><interface><requires lib=\"gt"
  "k+\" version=\"3.12\"/><requires lib=\"libxfce4ui-2\" version=\"4.12\"/"
  "><object class=\"GtkImage\" id=\"image1\"><property name=\"visible\">Tr"
  "ue</property><property name=\"can_focus\">False</property><property nam"
  "e=\"icon_name\">edit-clear-symbolic</property></object><object class=\""
  "GtkImage\" id=\"image2\"><property name=\"visible\">True</property><pro"
  "perty name=\"can_focus\">False</property><property name=\"icon_name\">d"
  "ocument-save-symbolic</property></object><object class=\"GtkImage\" id="
  "\"image3\"><property name=\"visible\">True</property><property name=\"c"
  "an_focus\">False</property><property name=\"icon_name\">list-remove-sym"
  "bolic</property></object><object class=\"GtkImage\" id=\"image4\"><prop"
  "erty name=\"visible\">True</property><property name=\"can_focus\">False"
  "</property><property name=\"icon_name\">list-remove-symbolic</property>"
  "</object><object class=\"GtkImage\" id=\"image5\"><property name=\"visi"
  "ble\">True</property><property name=\"can_focus\">False</property><prop"
  "erty name=\"icon_name\">window-close-symbolic</property></object><objec"
  "t class=\"GtkImage\" id=\"image7\"><property name=\"visible\">True</pro"
  "perty><property name=\"can_focus\">False</property><property name=\"ico"
  "n_name\">help-browser</property></object><object class=\"GtkImage\" id="
  "\"image8\"><property name=\"visible\">True</property><property name=\"c"
  "an_focus\">False</property><property name=\"icon_name\">window-close-sy"
  "mbolic</property></object><object class=\"GtkPopover\" id=\"session-inf"
  "o\"><property name=\"can_focus\">False</property><child><object class=\""
  "GtkLabel\"><property name=\"visible\">True</property><property name=\"c"
  "an_focus\">False</property><property name=\"margin_left\">12</property>"
  "<property name=\"margin_right\">12</property><property name=\"margin_to"
  "p\">12</property><property name=\"margin_bottom\">12</property><propert"
  "y name=\"label\" translatable=\"yes\">These applications are a part of "
  "the currently-running session,\nand can be saved now or when you log ou"
  "t.\nChanges below will only take effect when the session is saved.</pro"
  "perty><property name=\"use_markup\">True</property><property name=\"xal"
  "ign\">0</property></object></child></object><object class=\"XfceTitledD"
  "ialog\" id=\"xfce4_session_settings_dialog\"><property name=\"can_focus"
  "\">False</property><property name=\"title\" translatable=\"yes\">Sessio"
  "n and Startup</property><property name=\"window_position\">center-on-pa"
  "rent</property><property name=\"default_width\">600</property><property"
  " name=\"default_height\">500</property><property name=\"icon_name\">org"
  ".xfce.session</property><property name=\"type_hint\">dialog</property><"
  "child internal-child=\"vbox\"><object class=\"GtkBox\" id=\"dialog-vbox"
  "1\"><property name=\"visible\">True</property><property name=\"can_focu"
  "s\">False</property><property name=\"orientation\">vertical</property><"
  "property name=\"spacing\">2</property><child internal-child=\"action_ar"
  "ea\"><object class=\"GtkButtonBox\" id=\"dialog-action_area1\"><propert"
  "y name=\"visible\">True</property><property name=\"can_focus\">False</p"
  "roperty><child><object class=\"GtkButton\" id=\"button1\"><property nam"
  "e=\"label\" translatable=\"yes\">_Help</property><property name=\"use_a"
  "ction_appearance\">False</property><property name=\"visible\">True</pro"
  "perty><property name=\"can_focus\">True</property><property name=\"rece"
  "ives_default\">True</property><property name=\"image\">image7</property"
  "><property name=\"use_underline\">True</property></object><packing><pro"
  "perty name=\"expand\">False</property><property name=\"fill\">False</pr"
  "operty><property name=\"position\">0</property></packing></child><child"
  "><object class=\"GtkButton\" id=\"button2\"><property name=\"label\" tr"
  "anslatable=\"yes\">_Close</property><property name=\"use_action_appeara"
  "nce\">False</property><property name=\"visible\">True</property><proper"
  "ty name=\"can_focus\">True</property><property name=\"receives_default\""
  ">True</property><property name=\"image\">image5</property><property nam"
  "e=\"use_underline\">True</property></object><packing><property name=\"e"
  "xpand\">False</property><property name=\"fill\">False</property><proper"
  "ty name=\"position\">1</property></packing></child></object><packing><p"
  "roperty name=\"expand\">False</property><property name=\"fill\">True</p"
  "roperty><property name=\"pack_type\">end</property><property name=\"pos"
  "ition\">1</property></packing></child><child><object class=\"GtkNoteboo"
  "k\" id=\"plug-child\"><property name=\"visible\">True</property><proper"
  "ty name=\"can_focus\">True</property><property name=\"border_width\">6<"
  "/property><child><object class=\"GtkBox\" id=\"vbox1\"><property name=\""
  "visible\">True</property><property name=\"can_focus\">False</property><"
  "property name=\"valign\">start</property><property name=\"border_width\""
  ">12</property><property name=\"orientation\">vertical</property><proper"
  "ty name=\"spacing\">18</property><child><object class=\"GtkFrame\" id=\""
  "frame1\"><property name=\"visible\">True</property><property name=\"can"
  "_focus\">False</property><property name=\"label_xalign\">0</property><p"
  "roperty name=\"shadow_type\">none</property><child><object class=\"GtkA"
  "lignment\" id=\"alignment2\"><property name=\"visible\">True</property>"
  "<property name=\"can_focus\">False</property><property name=\"top_paddi"
  "ng\">6</property><property name=\"left_padding\">12</property><child><o"
  "bject class=\"GtkCheckButton\" id=\"chk_display_chooser\"><property nam"
  "e=\"label\" translatable=\"yes\">_Display chooser on login</property><p"
  "roperty name=\"use_action_appearance\">False</property><property name=\""
  "visible\">True</property><property name=\"can_focus\">True</property><p"
  "roperty name=\"receives_default\">False</property><property name=\"tool"
  "tip_text\" translatable=\"yes\">Display the session chooser every time "
  "Xfce starts</property><property name=\"use_underline\">True</property><"
  "property name=\"xalign\">0.5</property><property name=\"draw_indicator\""
  ">True</property></object></child></object></child><child type=\"label\""
  "><object class=\"GtkLabel\" id=\"label6\"><property name=\"visible\">Tr"
  "ue</property><property name=\"can_focus\">False</property><property nam"
  "e=\"label\" translatable=\"yes\">&lt;b&gt;Session Chooser&lt;/b&gt;</pr"
  "operty><property name=\"use_markup\">True</property></object></child></"
  "object><packing><property name=\"expand\">False</property><property nam"
  "e=\"fill\">True</property><property name=\"position\">0</property></pac"
  "king></child><child><object class=\"GtkFrame\" id=\"frame2\"><property "
  "name=\"visible\">True</property><property name=\"can_focus\">False</pro"
  "perty><property name=\"label_xalign\">0</property><property name=\"shad"
  "ow_type\">none</property><child><object class=\"GtkAlignment\" id=\"ali"
  "gnment3\"><property name=\"visible\">True</property><property name=\"ca"
  "n_focus\">False</property><property name=\"top_padding\">6</property><p"
  "roperty name=\"left_padding\">12</property><child><object class=\"GtkBo"
  "x\" id=\"vbox2\"><property name=\"visible\">True</property><property na"
  "me=\"can_focus\">False</property><property name=\"orientation\">vertica"
  "l</property><property name=\"spacing\">6</property><child><object class"
  "=\"GtkCheckButton\" id=\"chk_session_autosave\"><property name=\"label\""
  " translatable=\"yes\">Automatically save session on logo_ut</property><"
  "property name=\"use_action_appearance\">False</property><property name="
  "\"visible\">True</property><property name=\"can_focus\">True</property>"
  "<property name=\"receives_default\">False</property><property name=\"to"
  "oltip_text\" translatable=\"yes\">Always save the session when logging "
  "out</property><property name=\"use_underline\">True</property><property"
  " name=\"xalign\">0.5</property><property name=\"draw_indicator\">True</"
  "property></object><packing><property name=\"expand\">False</property><p"
  "roperty name=\"fill\">True</property><property name=\"position\">0</pro"
  "perty></packing></child><child><object class=\"GtkCheckButton\" id=\"ch"
  "k_logout_prompt\"><property name=\"label\" translatable=\"yes\">Pro_mpt"
  " on logout</property><property name=\"use_action_appearance\">False</pr"
  "operty><property name=\"visible\">True</property><property name=\"can_f"
  "ocus\">True</property><property name=\"receives_default\">False</proper"
  "ty><property name=\"tooltip_text\" translatable=\"yes\">Prompt for conf"
  "irmation when logging out</property><property name=\"use_underline\">Tr"
  "ue</property><property name=\"xalign\">0.5</property><property name=\"a"
  "ctive\">True</property><property name=\"draw_indicator\">True</property"
  "></object><packing><property name=\"expand\">False</property><property "
  "name=\"fill\">True</property><property name=\"position\">1</property></"
  "packing></child></object></child></object></child><child type=\"label\""
  "><object class=\"GtkLabel\" id=\"label7\"><property name=\"visible\">Tr"
  "ue</property><property name=\"can_focus\">False</property><property nam"
  "e=\"label\" translatable=\"yes\">&lt;b&gt;Logout Settings&lt;/b&gt;</pr"
  "operty><property name=\"use_markup\">True</property></object></child></"
  "object><packing><property name=\"expand\">True</property><property name"
  "=\"fill\">True</property><property name=\"position\">1</property></pack"
  "ing></child><child><object class=\"GtkFrame\" id=\"frame5\"><property n"
  "ame=\"visible\">True</property><property name=\"can_focus\">False</prop"
  "erty><property name=\"label_xalign\">0</property><property name=\"shado"
  "w_type\">none</property><child><object class=\"GtkAlignment\" id=\"alig"
  "nment1\"><property name=\"visible\">True</property><property name=\"can"
  "_focus\">False</property><property name=\"top_padding\">6</property><pr"
  "operty name=\"left_padding\">12</property><child><object class=\"GtkBox"
  "\" id=\"vbox6\"><property name=\"visible\">True</property><property nam"
  "e=\"can_focus\">False</property><property name=\"orientation\">vertical"
  "</property><property name=\"spacing\">6</property><child><object class="
  "\"GtkCheckButton\" id=\"chk_lock_screen\"><property name=\"label\" tran"
  "slatable=\"yes\">Lock screen be_fore sleep</property><property name=\"u"
  "se_action_appearance\">False</property><property name=\"visible\">True<"
  "/property><property name=\"can_focus\">True</property><property name=\""
  "receives_default\">False</property><property name=\"use_underline\">Tru"
  "e</property><property name=\"xalign\">0</property><property name=\"draw"
  "_indicator\">True</property></object><packing><property name=\"expand\""
  ">False</property><property name=\"fill\">True</property><property name="
  "\"position\">0</property></packing></child></object></child></object></"
  "child><child type=\"label\"><object class=\"GtkLabel\" id=\"label25\"><"
  "property name=\"visible\">True</property><property name=\"can_focus\">F"
  "alse</property><property name=\"label\" translatable=\"yes\">&lt;b&gt;S"
  "hutdown&lt;/b&gt;</property><property name=\"use_markup\">True</propert"
  "y></object></child></object><packing><property name=\"expand\">True</pr"
  "operty><property name=\"fill\">True</property><property name=\"position"
  "\">2</property></packing></child></object></child><child type=\"tab\"><"
  "object class=\"GtkLabel\" id=\"label1\"><property name=\"visible\">True"
  "</property><property name=\"can_focus\">False</property><property name="
  "\"label\" translatable=\"yes\">_General</property><property name=\"use_"
  "underline\">True</property></object><packing><property name=\"tab_fill\""
  ">False</property></packing></child><child><object class=\"GtkBox\" id=\""
  "vbox8\"><property name=\"visible\">True</property><property name=\"can_"
  "focus\">False</property><property name=\"border_width\">12</property><p"
  "roperty name=\"orientation\">vertical</property><child><object class=\""
  "GtkInfoBar\"><property name=\"visible\">True</property><property name=\""
  "can_focus\">False</property><child internal-child=\"action_area\"><obje"
  "ct class=\"GtkButtonBox\"><property name=\"can_focus\">False</property>"
  "<property name=\"spacing\">6</property><property name=\"layout_style\">"
  "end</property><child><object class=\"GtkButton\" id=\"btn_save_session\""
  "><property name=\"label\" translatable=\"yes\">Save Sess_ion</property>"
  "<property name=\"visible\">True</property><property name=\"can_focus\">"
  "True</property><property name=\"receives_default\">True</property><prop"
  "erty name=\"valign\">center</property><property name=\"image\">image2</"
  "property><property name=\"use_underline\">True</property></object><pack"
  "ing><property name=\"expand\">False</property><property name=\"fill\">T"
  "rue</property><property name=\"pack_type\">end</property><property name"
  "=\"position\">0</property></packing></child><child><object class=\"GtkM"
  "enuButton\"><property name=\"visible\">True</property><property name=\""
  "can_focus\">True</property><property name=\"receives_default\">True</pr"
  "operty><property name=\"popover\">session-info</property><child><object"
  " class=\"GtkImage\"><property name=\"visible\">True</property><property"
  " name=\"can_focus\">False</property><property name=\"icon_name\">dialog"
  "-information-symbolic</property></object></child><style><class name=\"c"
  "ircular\"/></style></object><packing><property name=\"expand\">False</p"
  "roperty><property name=\"fill\">True</property><property name=\"pack_ty"
  "pe\">end</property><property name=\"position\">1</property><property na"
  "me=\"non_homogeneous\">True</property></packing></child></object><packi"
  "ng><property name=\"expand\">False</property><property name=\"fill\">Fa"
  "lse</property><property name=\"position\">0</property></packing></child"
  "><child internal-child=\"content_area\"><object class=\"GtkBox\"><prope"
  "rty name=\"can_focus\">False</property><child><object class=\"GtkLabel\""
  " id=\"label_active_session\"><property name=\"visible\">True</property>"
  "<property name=\"can_focus\">False</property><property name=\"label\" t"
  "ranslatable=\"yes\">Currently active session: &lt;b&gt;Default&lt;/b&gt"
  ";</property><property name=\"use_markup\">True</property></object><pack"
  "ing><property name=\"expand\">False</property><property name=\"fill\">T"
  "rue</property><property name=\"position\">0</property></packing></child"
  "></object><packing><property name=\"expand\">False</property><property "
  "name=\"fill\">False</property><property name=\"position\">0</property><"
  "/packing></child><child><placeholder/></child></object><packing><proper"
  "ty name=\"expand\">False</property><property name=\"fill\">True</proper"
  "ty><property name=\"position\">1</property></packing></child><child><ob"
  "ject class=\"GtkScrolledWindow\" id=\"scrolledwindow4\"><property name="
  "\"visible\">True</property><property name=\"can_focus\">True</property>"
  "<property name=\"shadow_type\">etched-in</property><child><object class"
  "=\"GtkTreeView\" id=\"treeview_clients\"><property name=\"visible\">Tru"
  "e</property><property name=\"can_focus\">True</property><child internal"
  "-child=\"selection\"><object class=\"GtkTreeSelection\" id=\"treeview-s"
  "election2\"/></child></object></child></object><packing><property name="
  "\"expand\">True</property><property name=\"fill\">True</property><prope"
  "rty name=\"position\">2</property></packing></child><child><object clas"
  "s=\"GtkBox\" id=\"hbox7\"><property name=\"visible\">True</property><pr"
  "operty name=\"can_focus\">False</property><property name=\"spacing\">6<"
  "/property><child><object class=\"GtkBox\"><property name=\"visible\">Tr"
  "ue</property><property name=\"can_focus\">False</property><child><place"
  "holder/></child><child><object class=\"GtkButton\" id=\"btn_quit_client"
  "\"><property name=\"label\" translatable=\"yes\">_Quit Program</propert"
  "y><property name=\"visible\">True</property><property name=\"can_focus\""
  ">True</property><property name=\"receives_default\">True</property><pro"
  "perty name=\"image\">image3</property><property name=\"use_underline\">"
  "True</property></object><packing><property name=\"expand\">False</prope"
  "rty><property name=\"fill\">True</property><property name=\"position\">"
  "1</property></packing></child><style><class name=\"inline-toolbar\"/></"
  "style></object><packing><property name=\"expand\">True</property><prope"
  "rty name=\"fill\">True</property><property name=\"pack_type\">end</prop"
  "erty><property name=\"position\">3</property></packing></child></object"
  "><packing><property name=\"expand\">False</property><property name=\"fi"
  "ll\">True</property><property name=\"position\">3</property></packing><"
  "/child></object><packing><property name=\"position\">1</property></pack"
  "ing></child><child type=\"tab\"><object class=\"GtkLabel\" id=\"label3\""
  "><property name=\"visible\">True</property><property name=\"can_focus\""
  ">False</property><property name=\"label\" translatable=\"yes\">Current "
  "Sessio_n</property><property name=\"use_underline\">True</property></ob"
  "ject><packing><property name=\"position\">1</property><property name=\""
  "tab_fill\">False</property></packing></child><child><object class=\"Gtk"
  "Box\"><property name=\"visible\">True</property><property name=\"can_fo"
  "cus\">False</property><property name=\"border_width\">12</property><pro"
  "perty name=\"orientation\">vertical</property><child><object class=\"Gt"
  "kBox\"><property name=\"visible\">True</property><property name=\"can_f"
  "ocus\">False</property><property name=\"orientation\">vertical</propert"
  "y><child><object class=\"GtkScrolledWindow\" id=\"saved-sessions\"><pro"
  "perty name=\"visible\">True</property><property name=\"can_focus\">True"
  "</property><property name=\"shadow_type\">in</property><child><object c"
  "lass=\"GtkTreeView\" id=\"saved-sessions-list\"><property name=\"visibl"
  "e\">True</property><property name=\"can_focus\">True</property><child i"
  "nternal-child=\"selection\"><object class=\"GtkTreeSelection\"/></child"
  "></object></child></object><packing><property name=\"expand\">True</pro"
  "perty><property name=\"fill\">True</property><property name=\"position\""
  ">0</property></packing></child><child><object class=\"GtkBox\"><propert"
  "y name=\"visible\">True</property><property name=\"can_focus\">False</p"
  "roperty><child><object class=\"GtkBox\"><property name=\"visible\">True"
  "</property><property name=\"can_focus\">False</property><property name="
  "\"orientation\">vertical</property><child><object class=\"GtkButton\" i"
  "d=\"btn_delete_session\"><property name=\"label\" translatable=\"yes\">"
  "_Remove</property><property name=\"visible\">True</property><property n"
  "ame=\"can_focus\">True</property><property name=\"receives_default\">Tr"
  "ue</property><property name=\"tooltip_text\" translatable=\"yes\">Delet"
  "e the selected session</property><property name=\"image\">image4</prope"
  "rty><property name=\"use_underline\">True</property></object><packing><"
  "property name=\"expand\">False</property><property name=\"fill\">True</"
  "property><property name=\"position\">0</property></packing></child></ob"
  "ject><packing><property name=\"expand\">False</property><property name="
  "\"fill\">True</property><property name=\"position\">0</property></packi"
  "ng></child><child><object class=\"GtkBox\"><property name=\"visible\">T"
  "rue</property><property name=\"can_focus\">False</property><property na"
  "me=\"orientation\">vertical</property><child><object class=\"GtkButton\""
  " id=\"btn_clear_sessions\"><property name=\"label\" translatable=\"yes\""
  ">Clear Save_d Sessions</property><property name=\"visible\">True</prope"
  "rty><property name=\"can_focus\">True</property><property name=\"receiv"
  "es_default\">True</property><property name=\"halign\">end</property><pr"
  "operty name=\"valign\">center</property><property name=\"image\">image1"
  "</property><property name=\"use_underline\">True</property><property na"
  "me=\"always_show_image\">True</property></object><packing><property nam"
  "e=\"expand\">False</property><property name=\"fill\">True</property><pr"
  "operty name=\"position\">0</property></packing></child></object><packin"
  "g><property name=\"expand\">False</property><property name=\"fill\">Fal"
  "se</property><property name=\"pack_type\">end</property><property name="
  "\"position\">1</property></packing></child><style><class name=\"inline-"
  "toolbar\"/></style></object><packing><property name=\"expand\">False</p"
  "roperty><property name=\"fill\">True</property><property name=\"positio"
  "n\">1</property></packing></child></object><packing><property name=\"ex"
  "pand\">True</property><property name=\"fill\">True</property><property "
  "name=\"position\">0</property></packing></child></object><packing><prop"
  "erty name=\"position\">3</property></packing></child><child type=\"tab\""
  "><object class=\"GtkLabel\"><property name=\"visible\">True</property><"
  "property name=\"can_focus\">False</property><property name=\"label\" tr"
  "anslatable=\"yes\">Saved _Sessions</property><property name=\"use_under"
  "line\">True</property></object><packing><property name=\"position\">1</"
  "property><property name=\"tab_fill\">False</property></packing></child>"
  "<child><object class=\"GtkBox\" id=\"vbox3\"><property name=\"visible\""
  ">True</property><property name=\"can_focus\">False</property><property "
  "name=\"border_width\">12</property><property name=\"orientation\">verti"
  "cal</property><property name=\"spacing\">18</property><child><object cl"
  "ass=\"GtkFrame\" id=\"frame3\"><property name=\"visible\">True</propert"
  "y><property name=\"can_focus\">False</property><property name=\"label_x"
  "align\">0</property><property name=\"shadow_type\">none</property><chil"
  "d><object class=\"GtkAlignment\" id=\"alignment4\"><property name=\"vis"
  "ible\">True</property><property name=\"can_focus\">False</property><pro"
  "perty name=\"top_padding\">6</property><property name=\"left_padding\">"
  "12</property><child><object class=\"GtkBox\" id=\"vbox4\"><property nam"
  "e=\"visible\">True</property><property name=\"can_focus\">False</proper"
  "ty><property name=\"orientation\">vertical</property><property name=\"s"
  "pacing\">6</property><child><object class=\"GtkCheckButton\" id=\"chk_c"
  "ompat_gnome\"><property name=\"label\" translatable=\"yes\">Launch GN_O"
  "ME services on startup</property><property name=\"use_action_appearance"
  "\">False</property><property name=\"visible\">True</property><property "
  "name=\"can_focus\">True</property><property name=\"receives_default\">F"
  "alse</property><property name=\"tooltip_text\" translatable=\"yes\">Sta"
  "rt GNOME services, such as gnome-keyring</property><property name=\"use"
  "_underline\">True</property><property name=\"xalign\">0.5</property><pr"
  "operty name=\"draw_indicator\">True</property></object><packing><proper"
  "ty name=\"expand\">False</property><property name=\"fill\">True</proper"
  "ty><property name=\"position\">0</property></packing></child><child><ob"
  "ject class=\"GtkCheckButton\" id=\"chk_compat_kde\"><property name=\"la"
  "bel\" translatable=\"yes\">Launch _KDE services on startup</property><p"
  "roperty name=\"use_action_appearance\">False</property><property name=\""
  "visible\">True</property><property name=\"can_focus\">True</property><p"
  "roperty name=\"receives_default\">False</property><property name=\"tool"
  "tip_text\" translatable=\"yes\">Start KDE services, such as kdeinit</pr"
  "operty><property name=\"use_underline\">True</property><property name=\""
  "xalign\">0.5</property><property name=\"draw_indicator\">True</property"
  "></object><packing><property name=\"expand\">False</property><property "
  "name=\"fill\">True</property><property name=\"position\">1</property></"
  "packing></child></object></child></object></child><child type=\"label\""
  "><object class=\"GtkLabel\" id=\"label8\"><property name=\"visible\">Tr"
  "ue</property><property name=\"can_focus\">False</property><property nam"
  "e=\"label\" translatable=\"yes\">&lt;b&gt;Compatibility&lt;/b&gt;</prop"
  "erty><property name=\"use_markup\">True</property></object></child></ob"
  "ject><packing><property name=\"expand\">False</property><property name="
  "\"fill\">True</property><property name=\"position\">0</property></packi"
  "ng></child><child><object class=\"GtkFrame\" id=\"frame4\"><property na"
  "me=\"visible\">True</property><property name=\"can_focus\">False</prope"
  "rty><property name=\"label_xalign\">0</property><property name=\"shadow"
  "_type\">none</property><child><object class=\"GtkAlignment\" id=\"align"
  "ment5\"><property name=\"visible\">True</property><property name=\"can_"
  "focus\">False</property><property name=\"top_padding\">6</property><pro"
  "perty name=\"left_padding\">12</property><child><object class=\"GtkChec"
  "kButton\" id=\"chk_enable_tcp\"><property name=\"label\" translatable=\""
  "yes\">Manage _remote applications</property><property name=\"use_action"
  "_appearance\">False</property><property name=\"visible\">True</property"
  "><property name=\"can_focus\">True</property><property name=\"receives_"
  "default\">False</property><property name=\"tooltip_text\" translatable="
  "\"yes\">Manage remote applications over the network (this may be a secu"
  "rity risk)</property><property name=\"use_underline\">True</property><p"
  "roperty name=\"xalign\">0.5</property><property name=\"draw_indicator\""
  ">True</property></object></child></object></child><child type=\"label\""
  "><object class=\"GtkLabel\" id=\"label9\"><property name=\"visible\">Tr"
  "ue</property><property name=\"can_focus\">False</property><property nam"
  "e=\"label\" translatable=\"yes\">&lt;b&gt;Security&lt;/b&gt;</property>"
  "<property name=\"use_markup\">True</property></object></child></object>"
  "<packing><property name=\"expand\">False</property><property name=\"fil"
  "l\">True</property><property name=\"position\">1</property></packing></"
  "child></object><packing><property name=\"position\">3</property></packi"
  "ng></child><child type=\"tab\"><object class=\"GtkLabel\" id=\"label4\""
  "><property name=\"visible\">True</property><property name=\"can_focus\""
  ">False</property><property name=\"label\" translatable=\"yes\">Ad_vance"
  "d</property><property name=\"use_underline\">True</property></object><p"
  "acking><property name=\"position\">3</property><property name=\"tab_fil"
  "l\">False</property></packing></child><child><placeholder/></child><chi"
  "ld type=\"tab\"><placeholder/></child></object><packing><property name="
  "\"expand\">True</property><property name=\"fill\">True</property><prope"
  "rty name=\"position\">0</property></packing></child></object></child><a"
  "ction-widgets><action-widget response=\"-11\">button1</action-widget><a"
  "ction-widget response=\"-7\">button2</action-widget></action-widgets></"
  "object><object class=\"GtkDialog\" id=\"dialog_saving\"><property name="
  "\"can_focus\">False</property><property name=\"border_width\">5</proper"
  "ty><property name=\"title\" translatable=\"yes\">Saving Session</proper"
  "ty><property name=\"resizable\">False</property><property name=\"window"
  "_position\">center-on-parent</property><property name=\"destroy_with_pa"
  "rent\">True</property><property name=\"icon_name\">xfce4-session</prope"
  "rty><property name=\"type_hint\">dialog</property><property name=\"skip"
  "_taskbar_hint\">True</property><property name=\"skip_pager_hint\">True<"
  "/property><property name=\"transient_for\">xfce4_session_settings_dialo"
  "g</property><child><placeholder/></child><child internal-child=\"vbox\""
  "><object class=\"GtkBox\" id=\"dialog-vbox2\"><property name=\"visible\""
  ">True</property><property name=\"can_focus\">False</property><property "
  "name=\"orientation\">vertical</property><property name=\"spacing\">2</p"
  "roperty><child internal-child=\"action_area\"><object class=\"GtkButton"
  "Box\" id=\"dialog-action_area2\"><property name=\"visible\">True</prope"
  "rty><property name=\"can_focus\">False</property><property name=\"layou"
  "t_style\">end</property><child><object class=\"GtkButton\" id=\"button3"
  "\"><property name=\"label\">_Close</property><property name=\"use_actio"
  "n_appearance\">False</property><property name=\"visible\">True</propert"
  "y><property name=\"can_focus\">True</property><property name=\"receives"
  "_default\">True</property><property name=\"image\">image8</property><pr"
  "operty name=\"use_underline\">True</property></object><packing><propert"
  "y name=\"expand\">False</property><property name=\"fill\">False</proper"
  "ty><property name=\"position\">0</property></packing></child></object><"
  "packing><property name=\"expand\">False</property><property name=\"fill"
  "\">True</property><property name=\"pack_type\">end</property><property "
  "name=\"position\">0</property></packing></child><child><object class=\""
  "GtkBox\" id=\"vbox5\"><property name=\"visible\">True</property><proper"
  "ty name=\"can_focus\">False</property><property name=\"border_width\">6"
  "</property><property name=\"orientation\">vertical</property><property "
  "name=\"spacing\">6</property><child><object class=\"GtkBox\" id=\"hbox8"
  "\"><property name=\"visible\">True</property><property name=\"can_focus"
  "\">False</property><property name=\"spacing\">12</property><child><obje"
  "ct class=\"GtkImage\" id=\"image6\"><property name=\"visible\">True</pr"
  "operty><property name=\"can_focus\">False</property><property name=\"ic"
  "on_name\">dialog-information</property><property name=\"icon_size\">6</"
  "property></object><packing><property name=\"expand\">False</property><p"
  "roperty name=\"fill\">True</property><property name=\"position\">0</pro"
  "perty></packing></child><child><object class=\"GtkLabel\" id=\"label10\""
  "><property name=\"visible\">True</property><property name=\"can_focus\""
  ">False</property><property name=\"label\" translatable=\"yes\">Your ses"
  "sion is being saved.  If you do not wish to wait, you may close this wi"
  "ndow.</property><property name=\"wrap\">True</property><property name=\""
  "xalign\">0</property></object><packing><property name=\"expand\">True</"
  "property><property name=\"fill\">True</property><property name=\"positi"
  "on\">1</property></packing></child></object><packing><property name=\"e"
  "xpand\">False</property><property name=\"fill\">True</property><propert"
  "y name=\"position\">0</property></packing></child><child><object class="
  "\"GtkProgressBar\" id=\"progress_save_session\"><property name=\"visibl"
  "e\">True</property><property name=\"can_focus\">False</property></objec"
  "t><packing><property name=\"expand\">False</property><property name=\"f"
  "ill\">True</property><property name=\"position\">1</property></packing>"
  "</child></object><packing><property name=\"expand\">False</property><pr"
  "operty name=\"fill\">True</property><property name=\"position\">1</prop"
  "erty></packing></child></object></child><action-widgets><action-widget "
  "response=\"0\">button3</action-widget></action-widgets></object></inter"
  "face>"
};

static const unsigned xfce4_session_settings_ui_length = 28405u;

